﻿namespace U2UC.WinUni.SpeechSample.Services
{
    using System;
    using System.Collections.Generic;
    using Windows.Foundation;
    using Windows.Media.SpeechSynthesis;
    using Windows.UI.Core;
    using Windows.UI.Xaml;
    using Windows.UI.Xaml.Controls;
    using Windows.UI.Xaml.Media;

    /// <summary>
    /// Provides speech services.
    /// </summary>
    public class Voice
    {
        private MediaElement mediaElement;

        public Voice()
        {
            DependencyObject rootGrid = VisualTreeHelper.GetChild(Window.Current.Content, 0);
            this.mediaElement = (MediaElement)VisualTreeHelper.GetChild(rootGrid, 0) as MediaElement;
        }

        /// <summary>
        /// Returns the list with all the voices on the device.
        /// </summary>
        public static IReadOnlyList<VoiceInformation> AllVoices
        {
            get
            {
                return SpeechSynthesizer.AllVoices;
            }
        }

        /// <summary>
        /// Gets the MediaElement that was injected into the page. 
        /// </summary>
        private MediaElement MediaElement
        {
            get
            {
                return this.mediaElement;
            }
        }
        /// <summary>
        /// Creates a text stream from a string.
        /// </summary>
        public void Say(string text, int voice = 0)
        {
            var synthesizer = new SpeechSynthesizer();
            var voices = SpeechSynthesizer.AllVoices;
            synthesizer.Voice = voices[voice];

            var spokenStream = synthesizer.SynthesizeTextToStreamAsync(text);

            spokenStream.Completed += this.SpokenStreamCompleted;
        }

        /// <summary>
        /// Creates a text stream from an SSML string.
        /// </summary>
        public void SaySSML(string text, int voice = 0)
        {
            var synthesizer = new SpeechSynthesizer();
            var voices = SpeechSynthesizer.AllVoices;
            synthesizer.Voice = voices[voice];

            var spokenStream = synthesizer.SynthesizeSsmlToStreamAsync(text);

            spokenStream.Completed += this.SpokenStreamCompleted;
        }

        /// <summary>
        /// The spoken stream is ready.
        /// </summary>
        private async void SpokenStreamCompleted(IAsyncOperation<SpeechSynthesisStream> asyncInfo, AsyncStatus asyncStatus)
        {
            var mediaElement = this.MediaElement;

            // Make sure to be on the UI Thread.
            var results = asyncInfo.GetResults();
            await mediaElement.Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, new DispatchedHandler(
                () => { mediaElement.SetSource(results, results.ContentType); }));
        }
    }
}
