﻿namespace U2UC.WinUni.SpeechSample
{
    using Bing.Speech;
    using System;
    using U2UC.WinUni.SpeechSample.Services;
    using Windows.UI.Xaml;
    using Windows.UI.Xaml.Controls;

    public sealed partial class MainPage : Page
    {
        SpeechRecognizer SR;

        public MainPage()
        {
            this.InitializeComponent();

            var credentials = new SpeechAuthorizationParameters();
#error You need application credentials here.
            credentials.ClientId = "YourClientId";
            credentials.ClientSecret = "YourClientSecret";

            // The UI control, no event handlers needed.
            this.SpeechControl.SpeechRecognizer = new SpeechRecognizer("en-US", credentials);

            // The custom speech recognizer UI.
            SR = new SpeechRecognizer("en-US", credentials);
            SR.AudioCaptureStateChanged += SR_AudioCaptureStateChanged;
            SR.RecognizerResultReceived += SR_RecognizerResultReceived;

            // The speaker control.
            this.Voices.ItemsSource = Voice.AllVoices;
            this.Voices.SelectedIndex = 0;
        }

        /// <summary>
        /// Activates the speech control.
        /// </summary>
        private async void OpenUI_Click(object sender, RoutedEventArgs e)
        {
            // Always call RecognizeSpeechToTextAsync from inside
            // a try block because it calls a web service.
            try
            {
                var result = await this.SpeechControl.SpeechRecognizer.RecognizeSpeechToTextAsync();
                if (result.TextConfidence != SpeechRecognitionConfidence.Rejected)
                {
                    ResultText.Text = result.Text;
                    var voice = new Voice();
                    voice.Say(result.Text);
                }
                else
                {
                    ResultText.Text = "Sorry, I did not get that.";
                }
            }
            catch (Exception ex)
            {
                // Put error handling here.
            }
        }

        /// <summary>
        /// Starts a speech recognition session through the custom UI.
        /// </summary>
        private async void ListenButton_Click(object sender, RoutedEventArgs e)
        {
            // Always call RecognizeSpeechToTextAsync from inside
            // a try block because it calls a web service.
            try
            {
                // Start speech recognition.
                var result = await SR.RecognizeSpeechToTextAsync();

                // Write the result to the TextBlock.
                if (result.TextConfidence != SpeechRecognitionConfidence.Rejected)
                {
                    ResultText.Text = result.Text;

                    var voice = new Voice();
                    voice.Say(result.Text);
                }
                else
                {
                    ResultText.Text = "Sorry, I did not get that.";
                }
            }
            catch (Exception ex)
            {
                // If there's an exception, show the Type and Message.
                ResultText.Text = string.Format("{0}: {1}",
                    ex.GetType().ToString(), ex.Message);
            }
        }

        /// <summary>
        /// Cancels the current speech recognition session.
        /// </summary>
        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            SR.RequestCancelOperation();
        }

        /// <summary>
        /// Stop listening and start thinking.
        /// </summary>
        private void StopButton_Click(object sender, RoutedEventArgs e)
        {
            SR.StopListeningAndProcessAudio();
        }

        /// <summary>
        /// Update the speech recognition audio capture state.
        /// </summary>
        private void SR_AudioCaptureStateChanged(SpeechRecognizer sender, SpeechRecognitionAudioCaptureStateChangedEventArgs args)
        {
            this.Status.Text = args.State.ToString();
        }

        /// <summary>
        /// A result was received. Whether or not it is intermediary depends on the capture state.
        /// </summary>
        private void SR_RecognizerResultReceived(SpeechRecognizer sender, SpeechRecognitionResultReceivedEventArgs args)
        {
            if (args.Text != null)
            {
                this.ResultText.Text = args.Text;
            }
        }

        /// <summary>
        /// Says the result text in the selected voice.
        /// </summary>
        private void SpeakButton_Click(object sender, RoutedEventArgs e)
        {
            var voice = new Voice();
            voice.Say(this.ResultText.Text, this.Voices.SelectedIndex);
        }

        /// <summary>
        /// Demonstrates speaking SSML.
        /// </summary>
        private async void SSMLButton_Click(object sender, RoutedEventArgs e)
        {
            // Get the prepared text.
            var folder = Windows.ApplicationModel.Package.Current.InstalledLocation;
            folder = await folder.GetFolderAsync("Assets");
            var file = await folder.GetFileAsync("SSML_Sample.xml");
            var ssml = await Windows.Storage.FileIO.ReadTextAsync(file);

            // Say it.
            var voice = new Voice();
            voice.SaySSML(ssml, this.Voices.SelectedIndex);
        }
    }
}
