﻿namespace U2UC.WinUni.SpeechSample
{
    using System;
    using U2UC.WinUni.SpeechSample.Services;
    using Windows.Foundation;
    using Windows.Media.SpeechRecognition;
    using Windows.UI.Core;
    using Windows.UI.Xaml;
    using Windows.UI.Xaml.Controls;
    using Windows.UI.Xaml.Navigation;

    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();

            this.NavigationCacheMode = NavigationCacheMode.Required;

            this.Voices.ItemsSource = Voice.AllVoices;
            this.Voices.SelectedIndex = 0;

            this.SpeechInputBox.TextChanged += this.SpeechInputBox_TextChanged;
        }

        /// <summary>
        /// Opens the speech recognizer UI.
        /// </summary>
        private async void OpenUI_Click(object sender, RoutedEventArgs e)
        {
            SpeechRecognizer recognizer = new SpeechRecognizer();

            SpeechRecognitionTopicConstraint topicConstraint
                 = new SpeechRecognitionTopicConstraint(SpeechRecognitionScenario.Dictation, "Development");

            recognizer.Constraints.Add(topicConstraint);
            await recognizer.CompileConstraintsAsync(); // Required

            // Open the UI.
            var results = await recognizer.RecognizeWithUIAsync();
            if (results.Confidence != SpeechRecognitionConfidence.Rejected)
            {
                this.Result.Text = results.Text;
                // No need to call 'Voice.Say'. The control speaks itself.
            }
            else
            {
                this.Result.Text = "Sorry, I did not get that.";
            }
        }

        /// <summary>
        /// Starts a speech recognition session.
        /// </summary>
        private async void Listen_Click(object sender, RoutedEventArgs e)
        {
            this.Result.Text = "Listening...";

            SpeechRecognizer recognizer = new SpeechRecognizer();

            SpeechRecognitionTopicConstraint topicConstraint
                 = new SpeechRecognitionTopicConstraint(SpeechRecognitionScenario.Dictation, "Development");

            recognizer.Constraints.Add(topicConstraint);
            await recognizer.CompileConstraintsAsync(); // Required

            var recognition = recognizer.RecognizeAsync();
            recognition.Completed += this.Recognition_Completed;
        }

        /// <summary>
        /// Speech recognition completed.
        /// </summary>
        private async void Recognition_Completed(IAsyncOperation<SpeechRecognitionResult> asyncInfo, AsyncStatus asyncStatus)
        {
            var results = asyncInfo.GetResults();

            if (results.Confidence != SpeechRecognitionConfidence.Rejected)
            {
                await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, new DispatchedHandler(
                    () => { this.Result.Text = results.Text; ; }));

                var voice = new Voice();
                voice.Say(results.Text);
            }
            else
            {
                this.Result.Text = "Sorry, I did not get that.";
            }
        }

        /// <summary>
        /// Says the result text in the selected voice.
        /// </summary>
        private void Speak_Click(object sender, RoutedEventArgs e)
        {
            var voice = new Voice();
            voice.Say(this.Result.Text, this.Voices.SelectedIndex);
        }

        /// <summary>
        /// Demonstrates speaking SSML.
        /// </summary>
        private async void SSMLButton_Click(object sender, RoutedEventArgs e)
        {
            // Get the prepared text.
            var folder = Windows.ApplicationModel.Package.Current.InstalledLocation;
            folder = await folder.GetFolderAsync("Assets");
            var file = await folder.GetFileAsync("SSML_Sample.xml");
            var ssml = await Windows.Storage.FileIO.ReadTextAsync(file);

            // Say it.
            var voice = new Voice();
            voice.SaySSML(ssml, this.Voices.SelectedIndex);
        }

        // The text in the Cortana box was updated.
        private void SpeechInputBox_TextChanged(object sender, EventArgs e)
        {
            this.Result.Text = this.SpeechInputBox.Text;
        }
    }
}
